"""

"""
from forestmq.provider import Provider, Protocol


class ForestMQ:

    domain: str

    provider: Provider

    def __init__(self, *, domain: str):
        self.domain = domain
        tcp = Protocol.TCP
        self.provider = Provider(protocol=tcp, domain=self.domain)



from enum import Enum

import httpx


from forestmq.session import Session
from forestmq.exceptions import ProviderError, SessionError
from forestmq.logger import logger


class Protocol(Enum):
    TCP = 1
    AMQP = 2  # TODO


class Provider:

    protocol: Protocol
    session: Session
    domain: str

    def __init__(self, *, protocol: Protocol, domain: str):
        self.protocol = protocol
        self.domain = domain
        if self.protocol == Protocol.TCP:
            logger.debug("Using TCP protocol")
            self.session = Session(
                domain=domain,
                path="/provider",
            )
        else:
            raise ProviderError("ForestMQ Error: Must use TCP protocol")

    def send_msg_sync(self, message: dict) -> str:
        data = {
            "destroy": False,
            "message": message,
        }
        try:
            resp = self.session.send_msg_sync(message=data)
            self.session.close()
            return resp.json()
        except httpx.RequestError as e:
            raise SessionError(f"FORESTMQ Error: Failed to send message in session") from e

    async def send_msg(self, message: dict) -> str:
        data = {
            "destroy": False,
            "message": message,
        }
        try:
            resp = await self.session.send_msg(message=data)
            return resp.json()
        except httpx.RequestError as e:
            raise SessionError(f"FORESTMQ Error: Failed to send message in session") from e



from urllib.parse import urljoin
import json

import httpx

from forestmq.logger import logger


class Session:

    domain: str
    path: str
    url: str
    headers: dict
    client: httpx.Client

    def __init__(self, *, domain: str, path: str):
        self.domain = domain
        self.path = path
        self.client = httpx.Client()
        self.headers = {
            "Content-Type": "application/json",
        }
        self.url = urljoin(self.domain, self.path)

    def send_msg_sync(self, *, message: dict) -> httpx.Response:
        response = self.client.post(
            url=self.url,
            json=message,
            headers=self.headers,
        )
        response.raise_for_status()
        return response

    async def send_msg(self, *, message: dict) -> httpx.Response:
        async with httpx.AsyncClient() as client:
            response = await client.post(
                url=self.url,
                json=message,
            )
            response.raise_for_status()
            return response

    def close(self):
        self.client.close()



from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='forestmq',
    version='0.0.1',
    description='Python client for ForestMQ',
    packages=["forestmq"],
    classifiers=[
        'Development Status :: 3 - Alpha',
        "Programming Language :: Python :: 3.10",
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Operating System :: OS Independent",
    ],
    py_modules=["flask_jwt_router"],
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/josefdigital/forestmq-python",
    author="Josef Digital",
    author_email="contact@josef.digital",
    install_requires=[
        "httpx",
    ]
)